@@ -1,5 +1,6 @@
 import CoreFoundation
 import Foundation
+import os
 
 // MARK: - Debug Touch Counter (Debug builds only)
 
@@ -12,22 +13,55 @@ import Foundation
 // Required because C callbacks cannot capture Swift context
 @unsafe nonisolated(unsafe) private var gDeviceMonitor: DeviceMonitor?
 
+/// Flag to control whether callbacks should be processed.
+/// Access to this flag must be synchronized using `gCallbackLock`; it is *not* atomic by itself.
+/// This is checked FIRST in the callback, while holding the lock, before accessing gDeviceMonitor,
+/// to prevent race conditions during teardown where gDeviceMonitor might be nil or pointing to
+/// a deallocated object.
+@unsafe nonisolated(unsafe) private var gCallbackEnabled: Bool = false
+
+/// Lock to synchronize callback processing with stop/deinit operations.
+/// Uses os_unfair_lock for minimal overhead in high-frequency C callbacks.
+/// This is Apple's recommended lock for performance-critical short critical sections.
+/// Note: os_unfair_lock must not be moved in memory after initialization - safe here as a global.
+@unsafe nonisolated(unsafe) private var gCallbackLock = os_unfair_lock()
+
+/// Reference to a DeviceMonitor pending cleanup.
+/// When a DeviceMonitor is stopped, it's moved here to keep it alive until
+/// the framework's internal thread has completed any in-flight callbacks.
+/// This prevents EXC_BAD_ACCESS from accessing a deallocated object.
+@unsafe nonisolated(unsafe) private var gPendingCleanup: DeviceMonitor?
+
 // MARK: - C Callback Function
 
 @unsafe private let deviceContactCallback: MTContactCallbackFunction = {
     device, touches, numTouches, timestamp, frame in
+    
+    // CRITICAL: Check the enabled flag FIRST, before accessing gDeviceMonitor.
+    // This prevents accessing a nil or dangling pointer during teardown.
+    // We use the lock to synchronize with stop() which sets the flag to false.
+    unsafe os_unfair_lock_lock(&gCallbackLock)
+    guard unsafe gCallbackEnabled else {
+        unsafe os_unfair_lock_unlock(&gCallbackLock)
+        return 0
+    }
+    // Copy the monitor reference while holding the lock
+    guard let monitor = unsafe gDeviceMonitor,
+          let touches = unsafe touches
+    else {
+        unsafe os_unfair_lock_unlock(&gCallbackLock)
+        return 0
+    }
+    unsafe os_unfair_lock_unlock(&gCallbackLock)
+    
     #if DEBUG
-        touchCount += 1
+        unsafe touchCount += 1
         // Log sparingly to avoid performance impact
         if unsafe touchCount <= 5 || touchCount % 500 == 0 {
             Log.debug(unsafe "Touch callback #\(touchCount): \(numTouches) touches", category: .device)
         }
     #endif
 
-    guard let monitor = unsafe gDeviceMonitor,
-          let touches = unsafe touches
-    else { return 0 }
-
     let shouldConsume = unsafe monitor.handleContact(
         device: device,
         touches: touches,
@@ -56,9 +90,10 @@ class DeviceMonitor: TouchDeviceProviding {
     /// Delay between unregistering callbacks and stopping devices.
     /// This allows the MultitouchSupport framework's internal thread (mt_ThreadedMTEntry)
     /// to complete any in-flight callback processing before we stop devices.
-    /// Value determined empirically: 50ms is sufficient to avoid CFRelease(NULL) crashes
-    /// while keeping the stop operation reasonably fast.
-    static let frameworkCleanupDelay: TimeInterval = 0.05
+    /// Value determined empirically: 100ms is sufficient to avoid CFRelease(NULL) crashes
+    /// and EXC_BREAKPOINT exceptions while keeping the stop operation reasonably fast.
+    /// Increased from 50ms to handle rapid foreground/background toggling scenarios.
+    static let frameworkCleanupDelay: TimeInterval = 0.1
 
     // MARK: - Properties
 
@@ -69,26 +104,37 @@ class DeviceMonitor: TouchDeviceProviding {
     nonisolated(unsafe) private var registeredDevices: Set<UnsafeMutableRawPointer> = unsafe []
     private var isRunning = false
 
+    /// Lock to protect concurrent access to device state during stop/start operations
+    private let stateLock = NSLock()
+
     /// Tracks whether this instance owns the global callback reference
     private var ownsGlobalReference = false
 
     // MARK: - Lifecycle
 
     init() {
+        // Acquire lock to safely update global state
+        unsafe os_unfair_lock_lock(&gCallbackLock)
+        
+        // Release any previous pending cleanup reference
+        // The old instance has had time to complete cleanup by now
+        unsafe gPendingCleanup = nil
+        
         // Only take ownership of the global reference if no other instance owns it
         // This prevents test interference when multiple DeviceMonitor instances are created
         if unsafe gDeviceMonitor == nil {
             unsafe gDeviceMonitor = unsafe self
             unsafe ownsGlobalReference = true
         }
+        
+        unsafe os_unfair_lock_unlock(&gCallbackLock)
     }
 
     deinit {
+        // Ensure we've fully stopped monitoring and unregistered callbacks.
+        // All global cleanup (gDeviceMonitor, gPendingCleanup, etc.) is handled
+        // inside stop() under gCallbackLock to avoid taking locks in deinit.
         unsafe stop()
-        // Only clear the global reference if this instance owns it
-        if unsafe ownsGlobalReference && gDeviceMonitor === self {
-            unsafe gDeviceMonitor = nil
-        }
     }
 
     // MARK: - Public Interface
@@ -96,6 +142,9 @@ class DeviceMonitor: TouchDeviceProviding {
     /// Start monitoring the default multitouch device
     @unsafe @discardableResult
     func start() -> Bool {
+        unsafe stateLock.lock()
+        defer { unsafe stateLock.unlock() }
+
         guard unsafe !isRunning else { return true }
 
         Log.info("DeviceMonitor starting...", category: .device)
@@ -152,36 +201,75 @@ class DeviceMonitor: TouchDeviceProviding {
         Log.info("DeviceMonitor started with \(deviceCount) device(s)", category: .device)
 
         unsafe isRunning = true
+        
+        // Enable callbacks AFTER all devices are registered.
+        // This ensures gDeviceMonitor is fully set up before callbacks can access it.
+        unsafe os_unfair_lock_lock(&gCallbackLock)
+        unsafe gCallbackEnabled = true
+        unsafe os_unfair_lock_unlock(&gCallbackLock)
+        
         return true
     }
 
     /// Stop monitoring
     /// Safe to call even if start() was never called
     @unsafe func stop() {
+        unsafe stateLock.lock()
+
         // Safe to call when not running - just return early
-        guard unsafe isRunning else { return }
+        guard unsafe isRunning else {
+            unsafe stateLock.unlock()
+            return
+        }
+
+        // Mark as not running FIRST to prevent callbacks from processing new events
+        unsafe isRunning = false
+
+        // Copy the registered devices to a local variable so we can unlock before sleeping
+        let devicesToStop = unsafe registeredDevices
+        unsafe registeredDevices.removeAll()
+        unsafe self.device = nil
+
+        unsafe stateLock.unlock()
+
+        // CRITICAL: Disable callbacks and handle global cleanup under lock.
+        // This must happen BEFORE unregistering callbacks with the framework,
+        // because the framework's internal thread (mt_ThreadedMTEntry) may still
+        // have in-flight callbacks that could access gDeviceMonitor.
+        // The lock synchronizes with the callback to ensure no callback is
+        // accessing gDeviceMonitor while we're tearing down.
+        unsafe os_unfair_lock_lock(&gCallbackLock)
+        unsafe gCallbackEnabled = false
+        
+        // Handle global cleanup: if this instance owns the global reference,
+        // move it to gPendingCleanup to keep it alive until the next DeviceMonitor
+        // is created. This prevents EXC_BAD_ACCESS if the framework's internal
+        // thread still has a reference to this callback and tries to invoke it.
+        if unsafe ownsGlobalReference && gDeviceMonitor === self {
+            unsafe gPendingCleanup = gDeviceMonitor
+            unsafe gDeviceMonitor = nil
+            unsafe ownsGlobalReference = false
+        }
+        unsafe os_unfair_lock_unlock(&gCallbackLock)
 
         // IMPORTANT: Unregister callbacks FIRST, before stopping devices
         // This prevents the framework's internal thread (mt_ThreadedMTEntry)
-        // from receiving callbacks while we're stopping devices, which causes
-        // a race condition crash (CFRelease called with NULL / invalid address)
-        for unsafe deviceRef in unsafe registeredDevices {
+        // from receiving NEW callbacks while we're stopping devices.
+        for unsafe deviceRef in unsafe devicesToStop {
             unsafe MTUnregisterContactFrameCallback(deviceRef, deviceContactCallback)
         }
 
-        // Brief pause to allow the framework's internal thread to see the
-        // unregistered callbacks and complete any in-flight operations.
-        // Without this, the framework thread may still be processing a callback
-        // when we call MTDeviceStop, causing a race condition.
+        // Brief pause to allow the framework's internal thread to complete
+        // any in-flight operations AFTER we've disabled callbacks and unregistered.
+        // Even with callbacks disabled, we still need to wait for any callback
+        // that was already dispatched but hasn't checked the flag yet.
+        // This sleep is intentionally done with lock unlocked to avoid blocking other threads.
         unsafe Thread.sleep(forTimeInterval: Self.frameworkCleanupDelay)
 
         // Now safe to stop devices
-        for unsafe deviceRef in unsafe registeredDevices {
+        for unsafe deviceRef in unsafe devicesToStop {
             unsafe MTDeviceStop(deviceRef)
         }
-        unsafe registeredDevices.removeAll()
-        unsafe self.device = nil
-        unsafe isRunning = false
 
         Log.info("DeviceMonitor stopped", category: .device)
     }
@@ -196,6 +284,15 @@ class DeviceMonitor: TouchDeviceProviding {
         count: Int32,
         timestamp: Double
     ) -> Bool {
+        // Check if we're still running before processing - this prevents processing
+        // events during shutdown, which could cause race conditions with the
+        // MultitouchSupport framework's internal thread
+        unsafe stateLock.lock()
+        let stillRunning = unsafe isRunning
+        unsafe stateLock.unlock()
+
+        guard stillRunning else { return false }
+
         // Pass touch data to delegate for gesture processing
         unsafe delegate?.deviceMonitor(
             self, didReceiveTouches: touches, count: count, timestamp: timestamp)
