@@ -11,9 +11,15 @@ final class MultitouchManager: @unchecked Sendable {
     /// Delay after stopping before restarting devices during wake-from-sleep.
     /// This allows the MultitouchSupport framework's internal thread (mt_ThreadedMTEntry)
     /// to fully complete cleanup before we start new devices.
-    /// Value determined empirically: 100ms is sufficient to avoid CFRelease(NULL) crashes
-    /// caused by the framework accessing deallocated resources.
-    static let restartCleanupDelay: TimeInterval = 0.1
+    /// Value determined empirically: 250ms is sufficient to avoid CFRelease(NULL) crashes
+    /// caused by the framework accessing deallocated resources. Increased from 100ms to
+    /// handle rapid foreground/background toggling that can trigger multiple restart cycles.
+    static let restartCleanupDelay: TimeInterval = 0.25
+
+    /// Minimum delay between restart operations to prevent race conditions.
+    /// When multiple restart triggers occur in rapid succession, we debounce them
+    /// by waiting at least this long after the last restart completed.
+    static let minimumRestartInterval: TimeInterval = 0.3
 
     // MARK: - Properties
 
@@ -64,6 +70,11 @@ final class MultitouchManager: @unchecked Sendable {
     // Work item for debouncing restarts
     private var restartWorkItem: DispatchWorkItem?
 
+    // Restart synchronization to prevent race conditions from rapid foreground/background toggling
+    private let restartLock = NSLock()
+    private var isRestartInProgress = false
+    private var lastRestartCompletedTime: TimeInterval = 0
+
     // Processing queue
     private let gestureQueue = DispatchQueue(label: "com.middledrag.gesture", qos: .userInteractive)
 
@@ -151,9 +162,14 @@ final class MultitouchManager: @unchecked Sendable {
 
     /// Stop monitoring
     func stop() {
-        // Cancel any pending restart
+        // Clear restart state and cancel any pending restart work item.
+        // This must be done under lock to prevent data races with restart().
+        restartLock.lock()
         restartWorkItem?.cancel()
         restartWorkItem = nil
+        isRestartInProgress = false
+        lastRestartCompletedTime = 0
+        restartLock.unlock()
 
         // If not monitoring AND no wake observer (normal stopped state), just return
         // We must proceed if either isMonitoring OR wakeObserver exists (meaning we might be in restart delay)
@@ -173,6 +189,45 @@ final class MultitouchManager: @unchecked Sendable {
         // Using wakeObserver allows retry after failed restart (when isMonitoring=false)
         // because internalStop() sets isMonitoring=false before setupEventTap() runs
         guard wakeObserver != nil || isMonitoring else { return }
+
+        // Prevent concurrent restart operations - this is critical to avoid race conditions
+        // when rapid foreground/background toggling triggers multiple restart() calls.
+        // The MultitouchSupport framework's internal thread can crash (EXC_BREAKPOINT) if
+        // we attempt overlapping stop/start cycles.
+        restartLock.lock()
+
+        // If a restart is already in progress, just let it complete
+        if isRestartInProgress {
+            Log.debug("Restart already in progress, skipping duplicate request", category: .device)
+            restartLock.unlock()
+            return
+        }
+
+        // Check if we're restarting too quickly after a previous restart
+        let now = CACurrentMediaTime()
+        let timeSinceLastRestart = now - lastRestartCompletedTime
+        if lastRestartCompletedTime > 0 && timeSinceLastRestart < Self.minimumRestartInterval {
+            Log.debug(unsafe "Restart throttled: \(String(format: "%.3f", timeSinceLastRestart))s since last restart", category: .device)
+            // Schedule a delayed restart instead
+            restartWorkItem?.cancel()
+            let remainingDelay = Self.minimumRestartInterval - timeSinceLastRestart
+            let workItem = DispatchWorkItem { [weak self] in
+                self?.restart()
+            }
+            restartWorkItem = workItem
+            restartLock.unlock()
+            DispatchQueue.main.asyncAfter(deadline: .now() + remainingDelay, execute: workItem)
+            return
+        }
+
+        isRestartInProgress = true
+
+        // Cancel any pending restart work item while still holding the lock.
+        // This prevents data races with stop() which also accesses restartWorkItem.
+        restartWorkItem?.cancel()
+        restartWorkItem = nil
+        restartLock.unlock()
+
         Log.info("Restarting multitouch monitoring", category: .device)
 
         // Store current state
@@ -185,16 +240,16 @@ final class MultitouchManager: @unchecked Sendable {
         // framework's internal thread (mt_ThreadedMTEntry) to fully complete cleanup.
         // Without this delay, there's a race condition where the framework thread
         // may still be releasing resources when we try to start new devices,
-        // causing CFRelease(NULL) crashes.
-
-        // Cancel any pending restart to prevent race conditions with multiple rapid restarts
-        restartWorkItem?.cancel()
+        // causing CFRelease(NULL) crashes or EXC_BREAKPOINT exceptions.
 
         let workItem = DispatchWorkItem { [weak self] in
             self?.performRestart(wasEnabled: wasEnabled)
         }
 
+        // Store work item under lock to prevent data races
+        restartLock.lock()
         restartWorkItem = workItem
+        restartLock.unlock()
 
         // Use async dispatch to avoid blocking the main thread during wake.
         DispatchQueue.main.asyncAfter(
@@ -204,7 +259,10 @@ final class MultitouchManager: @unchecked Sendable {
     /// Performs the actual restart after the cleanup delay
     private func performRestart(wasEnabled: Bool) {
         // Verify we should still restart (manager may have been stopped during delay)
-        guard wakeObserver != nil else { return }
+        guard wakeObserver != nil else {
+            markRestartComplete()
+            return
+        }
 
         applyConfiguration()
         let eventTapSuccess = eventTapSetupFactory()
@@ -214,6 +272,7 @@ final class MultitouchManager: @unchecked Sendable {
             isMonitoring = false
             isEnabled = false
             removeSleepWakeObservers()
+            markRestartComplete()
             return
         }
 
@@ -230,11 +289,22 @@ final class MultitouchManager: @unchecked Sendable {
             isMonitoring = false
             isEnabled = false
             removeSleepWakeObservers()
+            markRestartComplete()
             return
         }
 
         isMonitoring = true
         isEnabled = wasEnabled
+        markRestartComplete()
+        Log.info("Multitouch monitoring restarted successfully", category: .device)
+    }
+
+    /// Mark the restart operation as complete and record the completion time
+    private func markRestartComplete() {
+        restartLock.lock()
+        isRestartInProgress = false
+        lastRestartCompletedTime = CACurrentMediaTime()
+        restartLock.unlock()
     }
 
     /// Internal stop without removing sleep/wake observers
@@ -559,14 +629,43 @@ extension MultitouchManager: DeviceMonitorDelegate {
         // CGEventSource.flagsState is thread-safe and can be called from any thread
         let modifierFlags = CGEventSource.flagsState(.hidSystemState)
 
+        // CRITICAL: Copy touch data synchronously during this callback.
+        // The touches pointer is only valid for the duration of this callback.
+        // The MultitouchSupport framework owns this memory and may free/reuse it
+        // immediately after the callback returns. We MUST copy the data before
+        // dispatching to the gesture queue.
+        let touchCount = Int(count)
+        let touchDataCopy: [MTTouch]
+        if touchCount > 0 {
+            let touchArray = unsafe touches.bindMemory(to: MTTouch.self, capacity: touchCount)
+            // Create a Swift array copy of the touch data
+            touchDataCopy = (0..<touchCount).map { unsafe touchArray[$0] }
+        } else {
+            touchDataCopy = []
+        }
+
         // Gesture recognition and finger counting is done inside processTouches
         // State updates happen in delegate callbacks dispatched to main thread
-        // Note: touches pointer is valid only during this callback, but processTouches
-        // copies the data it needs synchronously before returning
-        let touchesPtr = unsafe touches  // Capture in local to make intent clear
+        // IMPORTANT: We must call processTouches even with zero touches so the
+        // gesture recognizer can properly end gestures via stableFrameCount logic.
         gestureQueue.async { [weak self] in
-            unsafe self?.gestureRecognizer.processTouches(
-                touchesPtr, count: Int(count), timestamp: timestamp, modifierFlags: modifierFlags)
+            if touchDataCopy.isEmpty {
+                // Zero touches - still need to notify gesture recognizer so it can
+                // properly end active gestures via stableFrameCount mechanism
+                unsafe self?.gestureRecognizer.processTouches(
+                    UnsafeMutableRawPointer(bitPattern: 1)!, // Non-null placeholder, won't be dereferenced when count=0
+                    count: 0,
+                    timestamp: timestamp,
+                    modifierFlags: modifierFlags)
+            } else {
+                // Use withUnsafeBufferPointer to get a pointer to our copied data
+                unsafe touchDataCopy.withUnsafeBufferPointer { buffer in
+                    guard let baseAddress = buffer.baseAddress else { return }
+                    let rawPointer = unsafe UnsafeMutableRawPointer(mutating: baseAddress)
+                    unsafe self?.gestureRecognizer.processTouches(
+                        rawPointer, count: touchCount, timestamp: timestamp, modifierFlags: modifierFlags)
+                }
+            }
         }
     }
 }
